@@ -1,6 +1,5 @@
 """Trainer and configuration for MAPO with maximum likelihood-trained model."""
 from raylab.agents import trainer
-from raylab.agents.mapo.trainer import DEFAULT_MODULE
 from raylab.agents.model_based import ModelBasedTrainer
 from raylab.agents.sac.trainer import sac_config
 from raylab.policy.model_based.training_mixin import TrainingSpec
@@ -12,7 +11,7 @@ from .policy import MlMAPOTorchPolicy
 @trainer.option("losses/", help="Configurations for model and actor loss functions")
 @trainer.option(
     "losses/grad_estimator",
-    "SF",
+    default="PD",
     help="""Gradient estimator for optimizing expectations.
 
     Possible types include
@@ -22,13 +21,23 @@ from .policy import MlMAPOTorchPolicy
 )
 @trainer.option(
     "losses/model_samples",
-    4,
+    default=1,
     help="Number of next states to sample from the model when calculating the"
     " model-aware deterministic policy gradient",
 )
-@trainer.option("module", DEFAULT_MODULE, override=True)
+@trainer.option("module/type", default="ModelBasedSAC")
 @trainer.option("torch_optimizer/models/type", "Adam")
-@trainer.option("model_training", TrainingSpec().to_dict(), help=TrainingSpec.__doc__)
+@trainer.option(
+    "model_training", default=TrainingSpec().to_dict(), help=TrainingSpec.__doc__
+)
+@trainer.option(
+    "model_warmup",
+    default=TrainingSpec().to_dict(),
+    help="""Specifications for model warm-up.
+
+    Same configurations as 'model_training'.
+    """,
+)
 @trainer.option("evaluation_config/explore", False, override=True)
 @trainer.option("rollout_fragment_length", 25, override=True)
 @sac_config
