@@ -5,6 +5,7 @@ from raylab.agents.sac import SACTorchPolicy
 from raylab.policy import EnvFnMixin
 from raylab.policy import ModelTrainingMixin
 from raylab.policy.losses import MAPO
+from raylab.policy.losses import ModelEnsembleMLE
 from raylab.policy.losses import SPAML
 from raylab.pytorch.optim import build_optimizer
 
@@ -12,8 +13,6 @@ from raylab.pytorch.optim import build_optimizer
 class MAPOTorchPolicy(ModelTrainingMixin, EnvFnMixin, SACTorchPolicy):
     """Model-Aware Policy Optimization policy in PyTorch to use with RLlib."""
 
-    # pylint:disable=abstract-method
-
     def __init__(self, observation_space, action_space, config):
         super().__init__(observation_space, action_space, config)
         self._setup_model_loss()
@@ -26,24 +25,35 @@ class MAPOTorchPolicy(ModelTrainingMixin, EnvFnMixin, SACTorchPolicy):
 
         return MAPOTrainer.options
 
+    @property
+    def model_training_loss(self):
+        return self.loss_paml
+
+    @property
+    def model_warmup_loss(self):
+        return self.loss_mle
+
     def compile(self):
         super().compile()
-        self.loss_model.compile()
+        self.loss_paml.compile()
+        self.loss_mle.compile()
         self.loss_actor.compile()
 
     @override(ModelTrainingMixin)
     def optimize_model(self, *args, **kwargs):
         # pylint:disable=signature-differs
-        self.loss_model.alpha = self.module.alpha().item()
+        self.loss_paml.alpha = self.module.alpha().item()
         return super().optimize_model(*args, **kwargs)
 
     def _setup_model_loss(self):
-        self.loss_model = SPAML(
+        self.loss_paml = SPAML(
             self.module.models, self.module.actor, self.module.critics
         )
-        self.loss_model.gamma = self.config["gamma"]
-        self.loss_model.grad_estimator = self.config["losses"]["grad_estimator"]
-        self.loss_model.lambda_ = self.config["losses"]["lambda"]
+        self.loss_paml.gamma = self.config["gamma"]
+        self.loss_paml.grad_estimator = self.config["losses"]["grad_estimator"]
+        self.loss_paml.lambda_ = self.config["losses"]["lambda"]
+
+        self.loss_mle = ModelEnsembleMLE(self.module.models)
 
     @override(SACTorchPolicy)
     def _setup_actor_loss(self):
@@ -63,12 +73,12 @@ class MAPOTorchPolicy(ModelTrainingMixin, EnvFnMixin, SACTorchPolicy):
 
     @override(EnvFnMixin)
     def _set_reward_hook(self):
-        self.loss_model.set_reward_fn(self.reward_fn)
+        self.loss_paml.set_reward_fn(self.reward_fn)
         self.loss_actor.set_reward_fn(self.reward_fn)
 
     @override(EnvFnMixin)
     def _set_termination_hook(self):
-        self.loss_model.set_termination_fn(self.termination_fn)
+        self.loss_paml.set_termination_fn(self.termination_fn)
         self.loss_actor.set_termination_fn(self.termination_fn)
 
     @override(EnvFnMixin)
