@@ -6,24 +6,12 @@ from raylab.policy.model_based.training_mixin import TrainingSpec
 
 from .policy import MAPOTorchPolicy
 
-DEFAULT_MODULE = {
-    "type": "ModelBasedSAC",
-    "model": {
-        "network": {"units": (128, 128), "activation": "Swish"},
-        "ensemble_size": 1,
-        "input_dependent_scale": True,
-        "parallelize": False,
-        "residual": True,
-    },
-    "critic": {"double_q": True},
-}
-
 
 @trainer.configure
 @trainer.option("losses/", help="Configurations for model and actor loss functions")
 @trainer.option(
     "losses/grad_estimator",
-    "SF",
+    default="PD",
     help="""Gradient estimator for optimizing expectations.
 
     Possible types include
@@ -33,18 +21,28 @@ DEFAULT_MODULE = {
 )
 @trainer.option(
     "losses/lambda",
-    0.0,
+    default=0.0,
     help="Model KL regularization to avoid degenerate solutions (needs tuning)",
 )
 @trainer.option(
     "losses/model_samples",
-    4,
+    default=1,
     help="Number of next states to sample from the model when calculating the"
     " model-aware deterministic policy gradient",
 )
-@trainer.option("module", DEFAULT_MODULE, override=True)
-@trainer.option("torch_optimizer/models/type", "Adam")
-@trainer.option("model_training", TrainingSpec().to_dict(), help=TrainingSpec.__doc__)
+@trainer.option("module/type", default="ModelBasedSAC")
+@trainer.option("torch_optimizer/models/type", default="Adam")
+@trainer.option(
+    "model_training", default=TrainingSpec().to_dict(), help=TrainingSpec.__doc__
+)
+@trainer.option(
+    "model_warmup",
+    default=TrainingSpec().to_dict(),
+    help="""Specifications for model warm-up.
+
+    Same configurations as 'model_training'.
+    """,
+)
 @trainer.option("holdout_ratio", 0, override=True)
 @trainer.option("max_holdout", 0, override=True)
 @trainer.option("evaluation_config/explore", False, override=True)
