@@ -16,6 +16,7 @@ from raylab.utils.annotations import TensorDict
 from .abstract import Loss
 from .mixins import EnvFunctionsMixin
 from .mixins import UniformModelPriorMixin
+from .utils import dist_params_stats
 
 
 class MAPO(EnvFunctionsMixin, UniformModelPriorMixin, Loss):
@@ -81,7 +82,7 @@ class MAPO(EnvFunctionsMixin, UniformModelPriorMixin, Loss):
             "loss(actor)": loss.item(),
             "entropy": entropy.item(),
         }
-        stats.update(self.model_dist_info(dist_params))
+        stats.update(dist_params_stats(dist_params, name="model"))
         stats.update(policy_info)
         return loss, stats
 
@@ -89,12 +90,7 @@ class MAPO(EnvFunctionsMixin, UniformModelPriorMixin, Loss):
         policy = self._modules["policy"]
         dist_params = policy(obs)
         sample, logp = policy.dist.rsample(dist_params)
-        info = {}
-        info.update(
-            {"policy_mean_" + k: v.mean().item() for k, v in dist_params.items()}
-        )
-        info.update({"policy_max_" + k: v.max().item() for k, v in dist_params.items()})
-        info.update({"policy_min_" + k: v.min().item() for k, v in dist_params.items()})
+        info = dist_params_stats(dist_params, name="policy")
         return sample, logp, info
 
     def one_step_action_value_surrogate(
