@@ -41,6 +41,101 @@ class DataloaderSpec(DataClassJsonMixin):
     def __post_init__(self):
         assert self.batch_size > 0, "Model batch size must be positive"
 
+    def build_dataloader(self, train_tensors: TensorDict) -> DataLoader:
+        """Returns a dataloader for the given tensors based on specifications."""
+        dataset = TensorDictDataset(train_tensors)
+        sampler = RandomSampler(dataset, replacement=self.replacement)
+        return DataLoader(dataset, sampler=sampler, batch_size=self.batch_size)
+
+
+ModelSnapshot = collections.namedtuple("ModelSnapshot", "epoch loss state_dict")
+
+
+@dataclass
+class Evaluator:
+    """Evaluates models and saves snapshots.
+
+    Holds snapshots for each model. A snapshot contains the epoch in which the
+    model was evaluated, its validation loss, and its state dict.
+
+    Note:
+        Upon creation, evaluates models on validation data to set initial
+        snapshots.
+
+    Args:
+        models: the model ensemble
+        loss_fn: the loss function for model ensemble
+        improvement_threshold: Minimum expected relative improvement in model
+            validation loss
+        patience_epochs: Number of epochs to wait for any of the models to
+            improve on the validation dataset before early stopping. If None,
+            disables eary stopping.
+    """
+
+    models: nn.ModuleList
+    loss_fn: Loss
+    eval_tensors: TensorDict
+    improvement_threshold: float
+    patience_epochs: Optional[int]
+
+    def __post_init__(self):
+        eval_losses, _ = self.loss_fn(self.eval_tensors)
+        eval_losses = eval_losses.tolist()
+        self._snapshots = [
+            ModelSnapshot(epoch=-1, loss=loss, state_dict=copy.deepcopy(m.state_dict()))
+            for m, loss in zip(self.models, eval_losses)
+        ]
+
+    def validate(self, epoch: int) -> Tuple[bool, StatDict]:
+        """Evaluate models on holdout data and update snapshots.
+
+        Args:
+            epoch: the epoch number
+
+        Returns:
+            A tuple with two values: whether or not to early stop training based
+            on validation loss improvement and a dict with validation loss info
+        """
+        eval_losses, eval_info = self.loss_fn(self.eval_tensors)
+        eval_losses = eval_losses.tolist()
+        eval_info = {"eval/" + k: v for k, v in eval_info.items()}
+
+        self._update_snapshots(epoch, eval_losses)
+
+        patience_epochs = self.patience_epochs or float("inf")
+        early_stop = epoch - max(s.epoch for s in self._snapshots) >= patience_epochs
+        return early_stop, eval_info
+
+    def _update_snapshots(self, epoch: int, eval_losses: List[float]):
+        snapshots = self._snapshots
+        threshold = self.improvement_threshold
+
+        def updated_snapshot(model, snap, cur_loss):
+            if (snap.loss - cur_loss) / snap.loss > threshold:
+                return ModelSnapshot(
+                    epoch=epoch,
+                    loss=cur_loss,
+                    state_dict=copy.deepcopy(model.state_dict()),
+                )
+            return snap
+
+        self._snapshots = [
+            updated_snapshot(model=m, snap=s, cur_loss=l)
+            for m, s, l in zip(self.models, snapshots, eval_losses)
+        ]
+
+    def restore_models(self) -> List[float]:
+        """Restore models to the best performing parameters.
+
+        Returns:
+            A list with the validation performances of each model
+        """
+        losses = []
+        for idx, snap in enumerate(self._snapshots):
+            self.models[idx].load_state_dict(snap.state_dict)
+            losses += [snap.loss]
+        return losses
+
 
 @dataclass
 class TrainingSpec(DataClassJsonMixin):
@@ -94,94 +189,54 @@ class TrainingSpec(DataClassJsonMixin):
             not self.max_holdout or self.max_holdout >= 0
         ), "Maximum number of holdout samples must be non-negative"
 
+    def epochs(self) -> Iterator[int]:
+        """Returns an iterator over the epoch numbers based on specifications."""
+        return iter(range(self.max_epochs)) if self.max_epochs else itertools.count()
 
-ModelSnapshot = collections.namedtuple("ModelSnapshot", "epoch loss state_dict")
+    def terminate_epoch(self, start_time: float, model_steps: int) -> bool:
+        """Returns whether to terminate the epoch based on time and grad steps."""
+        max_time = self.max_time or float("inf")
+        max_grad_steps = self.max_grad_steps or float("inf")
 
+        return time.time() - start_time >= max_time or model_steps >= max_grad_steps
 
-@dataclass
-class Evaluator:
-    """Evaluates models and saves snapshots.
+    def train_eval_tensors(
+        self, samples: SampleBatch, batch_keys: List[str], tensor_map_fn: callable
+    ) -> Tuple[TensorDict, Optional[TensorDict]]:
+        """Returns a tensor dict dataset split into training and validation.
 
-    Holds snapshots for each model. A snapshot contains the epoch in which the
-    model was evaluated, its validation loss, and its state dict.
-
-    Note:
-        Upon creation, evaluates models on validation data to set initial
-        snapshots.
-
-    Args:
-        models: the model ensemble
-        loss_fn: the loss function for model ensemble
-        improvement_threshold: Minimum expected relative improvement in model
-            validation loss
-        patience_epochs: Number of epochs to wait for any of the models to
-            improve on the validation dataset before early stopping. If None,
-            disables eary stopping.
-    """
-
-    models: nn.ModuleList
-    loss_fn: Loss
-    eval_tensors: TensorDict
-    improvement_threshold: float
-    patience_epochs: Optional[int]
-
-    def __post_init__(self):
-        eval_losses, _ = self.loss_fn(self.eval_tensors)
-        eval_losses = eval_losses.tolist()
-        self._snapshots = [
-            ModelSnapshot(epoch=-1, loss=loss, state_dict=copy.deepcopy(m.state_dict()))
-            for m, loss in zip(self.models, eval_losses)
-        ]
-
-    def validate(self, epoch: int) -> Tuple[bool, StatDict]:
-        """Evaluate models on holdout data and update snapshots.
-
-        Args:
-            epoch: the epoch number
-
-        Returns:
-            A tuple with two values: whether or not to early stop training based
-            on validation loss improvement and a dict with validation loss info
+        Shuffles the samples before splitting them.
         """
-        eval_losses, eval_info = self.loss_fn(self.eval_tensors)
-        eval_losses = eval_losses.tolist()
-        eval_info = {"eval_" + k: v for k, v in eval_info.items()}
+        total_count = samples.count
+        holdout = int(total_count * self.holdout_ratio)
+        if self.max_holdout is not None:
+            holdout = min(holdout, self.max_holdout)
 
-        self._update_snapshots(epoch, eval_losses)
+        samples.shuffle()
+        train_data, eval_data = samples.slice(holdout, None), samples.slice(0, holdout)
 
-        patience_epochs = self.patience_epochs or float("inf")
-        early_stop = epoch - max(s.epoch for s in self._snapshots) >= patience_epochs
-        return early_stop, eval_info
+        train_tensors = {k: tensor_map_fn(train_data[k]) for k in batch_keys}
+        if eval_data.count == 0:
+            eval_tensors = None
+        else:
+            eval_tensors = {k: tensor_map_fn(eval_data[k]) for k in batch_keys}
 
-    def _update_snapshots(self, epoch: int, eval_losses: List[float]):
-        snapshots = self._snapshots
-        threshold = self.improvement_threshold
+        return train_tensors, eval_tensors
 
-        def updated_snapshot(model, snap, cur_loss):
-            if (snap.loss - cur_loss) / snap.loss > threshold:
-                return ModelSnapshot(
-                    epoch=epoch,
-                    loss=cur_loss,
-                    state_dict=copy.deepcopy(model.state_dict()),
-                )
-            return snap
+    def build_evaluator(
+        self, models: nn.ModuleList, eval_tensors: Optional[TensorDict], loss_fn: Loss
+    ) -> Optional[Evaluator]:
+        """Returns a model evaluator based on internal specifications."""
+        if not (eval_tensors and self.improvement_threshold is not None):
+            return None
 
-        self._snapshots = [
-            updated_snapshot(model=m, snap=s, cur_loss=l)
-            for m, s, l in zip(self.models, snapshots, eval_losses)
-        ]
-
-    def restore_models(self) -> List[float]:
-        """Restore models to the best performing parameters.
-
-        Returns:
-            A list with the validation performances of each model
-        """
-        losses = []
-        for idx, snap in enumerate(self._snapshots):
-            self.models[idx].load_state_dict(snap.state_dict)
-            losses += [snap.loss]
-        return losses
+        return Evaluator(
+            models=models,
+            loss_fn=loss_fn,
+            eval_tensors=eval_tensors,
+            improvement_threshold=self.improvement_threshold,
+            patience_epochs=self.patience_epochs,
+        )
 
 
 class ModelTrainingMixin(ABC):
@@ -255,8 +310,8 @@ class ModelTrainingMixin(ABC):
         loss_fn = self.model_warmup_loss if warmup else self.model_training_loss
         spec = self.model_warmup_spec if warmup else self.model_training_spec
 
-        train_tensors, eval_tensors = self._train_eval_tensors(
-            samples, loss_fn.batch_keys, spec
+        train_tensors, eval_tensors = spec.train_eval_tensors(
+            samples, loss_fn.batch_keys, self.convert_to_tensor
         )
         # Fit scalers for each model here
         for model in self.module.models:
@@ -264,8 +319,8 @@ class ModelTrainingMixin(ABC):
                 train_tensors[SampleBatch.CUR_OBS], train_tensors[SampleBatch.ACTIONS]
             )
 
-        dataloader = self._build_dataloader(train_tensors, spec=spec)
-        evaluator = self._setup_evaluator(eval_tensors, loss_fn=loss_fn, spec=spec)
+        dataloader = spec.dataloader.build_dataloader(train_tensors)
+        evaluator = spec.build_evaluator(self.module.models, eval_tensors, loss_fn)
 
         info = self._train_model_epochs(
             dataloader, loss_fn=loss_fn, spec=spec, evaluator=evaluator
@@ -274,7 +329,7 @@ class ModelTrainingMixin(ABC):
         if evaluator:
             eval_losses = evaluator.restore_models()
             info.update(
-                {f"eval_loss(models[{i}])": l for i, l in enumerate(eval_losses)}
+                {f"eval/loss(models[{i}])": l for i, l in enumerate(eval_losses)}
             )
         else:
             eval_losses = [np.nan for _ in self.module.models]
@@ -282,46 +337,6 @@ class ModelTrainingMixin(ABC):
         info.update(self.model_grad_info())
         return eval_losses, info
 
-    def _train_eval_tensors(
-        self, samples: SampleBatch, batch_keys: List[str], spec: TrainingSpec
-    ) -> Tuple[TensorDict, Optional[TensorDict]]:
-        total_count = samples.count
-        holdout = int(total_count * spec.holdout_ratio)
-        if spec.max_holdout is not None:
-            holdout = min(holdout, spec.max_holdout)
-
-        samples.shuffle()
-        train_data, eval_data = samples.slice(holdout, None), samples.slice(0, holdout)
-
-        train_tensors = {k: self.convert_to_tensor(train_data[k]) for k in batch_keys}
-        if eval_data.count == 0:
-            eval_tensors = None
-        else:
-            eval_tensors = {k: self.convert_to_tensor(eval_data[k]) for k in batch_keys}
-
-        return train_tensors, eval_tensors
-
-    @staticmethod
-    def _build_dataloader(train_tensors: TensorDict, spec: TrainingSpec) -> DataLoader:
-        spec = spec.dataloader
-        dataset = TensorDictDataset(train_tensors)
-        sampler = RandomSampler(dataset, replacement=spec.replacement)
-        return DataLoader(dataset, sampler=sampler, batch_size=spec.batch_size)
-
-    def _setup_evaluator(
-        self, eval_tensors: Optional[TensorDict], loss_fn: Loss, spec: TrainingSpec
-    ) -> Optional[Evaluator]:
-        if not (eval_tensors and spec.improvement_threshold is not None):
-            return None
-
-        return Evaluator(
-            models=self.module.models,
-            loss_fn=loss_fn,
-            eval_tensors=eval_tensors,
-            improvement_threshold=spec.improvement_threshold,
-            patience_epochs=spec.patience_epochs,
-        )
-
     def _train_model_epochs(
         self,
         dataloader: DataLoader,
@@ -334,13 +349,13 @@ class ModelTrainingMixin(ABC):
         start = time.time()
         early_stop = False
         epoch = -1
-        for epoch in self._model_epochs(spec):
+        for epoch in spec.epochs():
             for minibatch in dataloader:
                 with self.optimizers.optimize("models"):
                     losses, train_info = loss_fn(minibatch)
                     losses.sum().backward()
 
-                info.update({"train_" + k: v for k, v in train_info.items()})
+                info.update({"train/" + k: v for k, v in train_info.items()})
                 grad_steps += 1
                 if spec.max_grad_steps and grad_steps >= spec.max_grad_steps:
                     break
@@ -349,25 +364,14 @@ class ModelTrainingMixin(ABC):
                 early_stop, eval_info = evaluator.validate(epoch)
                 info.update(eval_info)
 
-            if early_stop or self._terminate_epoch(start, grad_steps, spec):
+            if early_stop or spec.terminate_epoch(start, grad_steps):
                 break
 
         info["model_epochs"] = epoch + 1
+        info["model_grad_steps"] = grad_steps
+        info["early_stop"] = early_stop
         return info
 
-    @staticmethod
-    def _model_epochs(spec: TrainingSpec) -> Iterator[int]:
-        return iter(range(spec.max_epochs)) if spec.max_epochs else itertools.count()
-
-    @staticmethod
-    def _terminate_epoch(
-        start_time: float, model_steps: int, spec: TrainingSpec,
-    ) -> bool:
-        max_time = spec.max_time or float("inf")
-        max_grad_steps = spec.max_grad_steps or float("inf")
-
-        return time.time() - start_time >= max_time or model_steps >= max_grad_steps
-
     @torch.no_grad()
     def model_grad_info(self) -> StatDict:
         """Returns the average gradient norm accross models."""
