@@ -9,11 +9,14 @@ import torch
 import torch.nn as nn
 from dataclasses_json import DataClassJsonMixin
 from gym.spaces import Box
+from torch import Tensor
 
 import raylab.pytorch.nn as nnx
 from raylab.pytorch.nn.init import initialize_
 from raylab.pytorch.nn.utils import get_activation
 
+from .utils import TensorStandardScaler
+
 
 @dataclass
 class StateMLPSpec(DataClassJsonMixin):
@@ -67,30 +70,60 @@ class StateActionMLPSpec(DataClassJsonMixin):
         activation: Nonlinearity following each linear layer
         delay_action: Whether to apply an initial preprocessing layer on the
             observation before concatenating the action to the input.
+        standard_scaler: Whether to transform the inputs of the NN using a
+            standard scaling procedure (subtract mean and divide by stddev). The
+            transformation mean and stddev should be fitted during training and
+            used for both training and evaluation.
     """
 
     units: List[int] = field(default_factory=list)
     activation: Optional[str] = None
     delay_action: bool = False
+    standard_scaler: bool = False
 
 
-class StateActionMLP(nnx.StateActionEncoder):
+class StateActionMLP(nn.Module):
     """Multilayer perceptron for encoding state-action inputs."""
 
     spec_cls = StateActionMLPSpec
 
     def __init__(self, obs_space: Box, action_space: Box, spec: StateActionMLPSpec):
+        super().__init__()
+        self.spec = spec
         obs_size = obs_space.shape[0]
         action_size = action_space.shape[0]
 
-        super().__init__(
+        self.encoder = nnx.StateActionEncoder(
             obs_size,
             action_size,
             units=spec.units,
             activation=spec.activation,
             delay_action=spec.delay_action,
         )
-        self.spec = spec
+        self.out_features = self.encoder.out_features
+
+        if self.spec.standard_scaler:
+            self.obs_scaler = TensorStandardScaler(obs_size)
+            self.act_scaler = TensorStandardScaler(action_size)
+        else:
+            self.obs_scaler = None
+            self.act_scaler = None
+
+    @torch.jit.export
+    def fit_scaler(self, obs: Tensor, act: Tensor):
+        """Fit each sub-scaler to the inputs."""
+        if self.obs_scaler is not None:
+            self.obs_scaler.fit(obs)
+        if self.act_scaler is not None:
+            self.act_scaler.fit(act)
+
+    def forward(self, obs: Tensor, act: Tensor) -> Tensor:
+        # pylint:disable=arguments-differ
+        if self.obs_scaler is not None:
+            obs = self.obs_scaler(obs)
+        if self.act_scaler is not None:
+            act = self.act_scaler(act)
+        return self.encoder(obs, act)
 
     def initialize_parameters(self, initializer_spec: dict):
         """Initialize all Linear models in the encoder.
@@ -104,7 +137,7 @@ class StateActionMLP(nnx.StateActionEncoder):
                 keyword arguments.
         """
         initializer = initialize_(activation=self.spec.activation, **initializer_spec)
-        self.apply(initializer)
+        self.encoder.apply(initializer)
 
 
 class MLP(nn.Module):
