@@ -3,6 +3,7 @@ from typing import Dict
 
 import torch
 import torch.nn as nn
+import torch.nn.functional as F
 from ray.rllib.utils import override
 
 from raylab.pytorch.nn.init import initialize_
@@ -29,17 +30,24 @@ class CategoricalParams(nn.Module):
 
 
 class NormalParams(nn.Module):
+    # pylint:disable=line-too-long
     """Produce Normal parameters.
 
     Initialized to be close to a standard Normal distribution.
-    """
 
-    __constants__ = {"LOG_STD_MAX", "LOG_STD_MIN"}
-    LOG_STD_MAX = 2
-    LOG_STD_MIN = -20
+    Utilizes bounded log_stddev as described in the 'Well behaved probabilistic
+    networks' appendix of `PETS`_
+
+    .. _`PETS`: https://papers.nips.cc/paper/7725-deep-reinforcement-learning-in-a-handful-of-trials-using-probabilistic-dynamics-models
+    """
+    # pylint:enable=line-too-long
 
     def __init__(
-        self, in_features: int, event_size: int, input_dependent_scale: bool = True
+        self,
+        in_features: int,
+        event_size: int,
+        input_dependent_scale: bool = True,
+        bound_parameters: bool = True,
     ):
         super().__init__()
         self.loc_module = nn.Linear(in_features, event_size)
@@ -47,15 +55,36 @@ class NormalParams(nn.Module):
             self.log_scale_module = nn.Linear(in_features, event_size)
         else:
             self.log_scale_module = LeafParameter(event_size)
+
+        max_logvar = torch.ones(event_size) / 2.0
+        min_logvar = -torch.ones(event_size) * 10.0
+        if bound_parameters:
+            self.max_logvar = nn.Parameter(max_logvar)
+            self.min_logvar = nn.Parameter(min_logvar)
+        else:
+            self.register_buffer("max_logvar", max_logvar)
+            self.register_buffer("min_logvar", min_logvar)
+
         self.apply(initialize_("orthogonal", gain=0.01))
 
     @override(nn.Module)
     def forward(self, inputs: torch.Tensor) -> Dict[str, torch.Tensor]:
         # pylint:disable=arguments-differ
         loc = self.loc_module(inputs)
+
         log_scale = self.log_scale_module(inputs)
-        scale = torch.clamp(log_scale, self.LOG_STD_MIN, self.LOG_STD_MAX).exp()
-        return {"loc": loc, "scale": scale}
+        max_logvar = self.max_logvar.expand_as(log_scale)
+        min_logvar = self.min_logvar.expand_as(log_scale)
+        log_scale = max_logvar - F.softplus(max_logvar - log_scale)
+        log_scale = min_logvar + F.softplus(log_scale - min_logvar)
+        scale = log_scale.exp()
+
+        return {
+            "loc": loc,
+            "scale": scale,
+            "max_logvar": max_logvar,
+            "min_logvar": min_logvar,
+        }
 
 
 class StdNormalParams(nn.Module):
